package usecase

import (
	"strconv"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/entity"
)

type BatchSeminarUsecase interface {
	AddBatchSeminar(bimbinganLog entity.BatchSeminar) (entity.BatchSeminar, error)
	GetBatchSeminar() ([]entity.BatchSeminar, error)
	DeleteBatchSeminar(id string) (entity.BatchSeminar, error)
	GetBatchSeminarCurrent() ([]entity.BatchSeminar, error)
	SetLiveBatch(id string) (entity.BatchSeminar, error)
	GetBatchSeminarLive() ([]entity.BatchSeminar, error)
}

type BatchSeminarUc struct {
	batchSeminarRepo BatchSeminarRepository
}

func NewBatchSeminarUseCase(batchSeminarRepo BatchSeminarRepository) *BatchSeminarUc {
	return &BatchSeminarUc{
		batchSeminarRepo: batchSeminarRepo,
	}
}

func (uc *BatchSeminarUc) AddBatchSeminar(batchseminar entity.BatchSeminar) (entity.BatchSeminar, error) {
	startDate, _ := time.Parse("2006-01-02T15:04:05Z", batchseminar.StartDate)
	month := startDate.Month()
	year := startDate.Year()

	if month <= 6 {
		batchseminar.PeriodSemester = "II"
		batchseminar.PeriodYear = strconv.Itoa(year-1) + "/" + strconv.Itoa(year)
	} else {
		batchseminar.PeriodSemester = "I"
		batchseminar.PeriodYear = strconv.Itoa(year) + "/" + strconv.Itoa(year+1)
	}

	newBatchSeminar, err := uc.batchSeminarRepo.NewBatchSeminar(batchseminar)

	if err != nil {
		return newBatchSeminar, err
	}

	return newBatchSeminar, nil

}

func (uc *BatchSeminarUc) GetBatchSeminar() ([]entity.BatchSeminar, error) {
	return uc.batchSeminarRepo.GetSeminarBatches()
}

func (uc *BatchSeminarUc) DeleteBatchSeminar(id string) (entity.BatchSeminar, error) {
	return uc.batchSeminarRepo.DeleteBatchSeminar(id)
}

func (uc *BatchSeminarUc) GetBatchSeminarCurrent() ([]entity.BatchSeminar, error) {
	return uc.batchSeminarRepo.GetBatchSeminarCurrent()
}

func (uc *BatchSeminarUc) SetLiveBatch(id string) (entity.BatchSeminar, error) {
	return uc.batchSeminarRepo.SetLiveBatch(id)
}

func (uc *BatchSeminarUc) GetBatchSeminarLive() ([]entity.BatchSeminar, error) {
	return uc.batchSeminarRepo.GetBatchSeminarLive()
}
