package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type MahasiswaDashboardHandler struct {
	dashboardMahasiswaUsecase usecase.DashboardMahasiswaUseCase
}

func NewMahasiswaDashboardHandler(cfg config.DashboardMahasiswaTransportConfig) *MahasiswaDashboardHandler {
	dashboardMahasiswaRepository := repository.NewDashboardMahasiswaRepository(cfg.DBWrite, cfg.DBRead)
	dashboardMahasiswaUsecase := usecase.NewDashboardMahasiswaUsecase(dashboardMahasiswaRepository)

	return &MahasiswaDashboardHandler{
		dashboardMahasiswaUsecase: dashboardMahasiswaUsecase,
	}

}

func (t *MahasiswaDashboardHandler) MountMahasiswa(group *echo.Group) {
	group.GET("/dashboard", t.GetStatusMahasiswa)
}

// GetStatusMahasiswa retrieves the status mahasiswa by NIM.
// @Summary Get status mahasiswa
// @Description Retrieves the status mahasiswa by NIM.
// @Tags Status Mahasiswa
// @Accept json
// @Produce json
// @Success 200 {object} entity.DashboardMahasiswaRet
// @Router /api/mahasiswa/dashboard [get]
func (t *MahasiswaDashboardHandler) GetStatusMahasiswa(c echo.Context) error {
	payload, ok := c.Get("authPayload").(utils.Payload)

	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	dashboardMahasiswa, err := t.dashboardMahasiswaUsecase.GetDashboardMahasiswa(payload.ID)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Status mahasiswa successfully retreived", dashboardMahasiswa[0]))
}
