package repository

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_ta/entity"
	"gorm.io/gorm"
)

type DashboardTimRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardTimRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardTimRepo {
	return &DashboardTimRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardTimRepo) GetDataDashboard(offset, size int) (output []entity.DataPendaftaran, err error) {
	var dataMahasiswa []entity.DataPendaftaran

	var dosen models.Pengguna
	var mahasiswa models.Pengguna
	var pendaftaran []models.PendaftaranTA
	var logBimbingan []models.BimbinganLogs

	result := repo.DBRead.Limit(size).Offset(offset).Find(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	for _, pendaftar := range pendaftaran {
		res := repo.DBRead.Where("id = ?", pendaftar.IdMahasiswa).Limit(1).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", pendaftar.IdDosen).Limit(1).Find(&dosen)
		if res.Error != nil {
			return nil, res.Error
		}

		data := entity.DataPendaftaran{
			IdPendaftaran:   pendaftar.ID,
			Nama:            mahasiswa.Nama,
			NIM:             mahasiswa.NIM,
			DosenPembimbing: dosen.Nama,
			Status:          "Pengajuan Topik",
		}

		if pendaftar.Status == "APPROVED" {
			data.Status = "Bimbingan"

			res := repo.DBRead.Where("id_mahasiswa = ?", pendaftar.IdMahasiswa).Find(&logBimbingan)
			if res.Error != nil {
				return nil, res.Error
			}
			if len(logBimbingan) >= 3 {
				data.Status = "Eligible Seminar Proposal"
				if len(logBimbingan) >= 6 {
					data.Status = "Eligible Sidang TA"
				}
			}
		}

		dataMahasiswa = append(dataMahasiswa, data)
	}

	return dataMahasiswa, nil

}
