package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
	"gorm.io/gorm"
)

type PendaftaranRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewPendaftaranRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *PendaftaranRepo {
	return &PendaftaranRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *PendaftaranRepo) NewPendaftaran(pendaftaran models.PendaftaranTA) (output models.PendaftaranTA, err error) {
	pendaftaran.ID = uuid.New().String()
	pendaftaran.WaktuPengiriman = time.Now()
	pendaftaran.Status = models.Pendaftaran_ta_status_enum("NOT_ASSIGNED")
	result := repo.DBWrite.Create(&pendaftaran)
	if result.Error != nil {
		return models.PendaftaranTA{}, result.Error
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) GetPendaftaran() (output []models.PendaftaranTA, err error) {
	var pendaftaran []models.PendaftaranTA
	result := repo.DBRead.Find(&pendaftaran)
	if result.Error != nil {
		return nil, err
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) DeletePendaftaran(id string) (models.PendaftaranTA, error) {
	batch := models.PendaftaranTA{ID: id}
	err := repo.DBWrite.Delete(&batch).Error

	if err != nil {
		return batch, err
	}

	return batch, err
}

func (repo *PendaftaranRepo) GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error) {
	var pendaftaran []models.PendaftaranTA
	result := repo.DBRead.Where("id_dosen = ?", id).Find(&pendaftaran)
	if result.Error != nil {
		return nil, err
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error) {
	var pendaftaran []models.PendaftaranTA
	var dosen models.Pengguna
	var data []entity.PendaftaranGetResponse
	var topik models.Topik

	res := repo.DBRead.Where("id_mahasiswa = ?", id).Find(&pendaftaran)
	if res.Error != nil {
		return data, res.Error
	}

	for _, pen := range pendaftaran {
		res = repo.DBRead.Where("id = ?", pen.IdDosen).Find(&dosen).Limit(1)
		if res.Error != nil {
			return data, res.Error
		}

		res = repo.DBRead.Where("id = ?", pen.IdTopik).Find(&topik).Limit(1)
		if res.Error != nil {
			return data, res.Error
		}

		var dataTemp = entity.PendaftaranGetResponse{
			Id:              pen.ID,
			Status:          string(pen.Status),
			InterviewAt:     pen.InterviewAt,
			DecidedAt:       pen.DecidedAt,
			WaktuPengiriman: pen.WaktuPengiriman,
			Judul:           topik.Judul,
			Deskripsi:       topik.Deskripsi,
			IdDosen:         pen.IdDosen,
			NamaDosen:       dosen.Nama,
			EmailDosen:      dosen.Email,
		}

		data = append(data, dataTemp)
	}

	if res.Error != nil {
		return data, err
	}
	return data, nil
}

func (repo *PendaftaranRepo) SetStatus(id string, val string) (output []models.PendaftaranTA, err error) {
	var pendaftaran models.PendaftaranTA
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.Status = models.Pendaftaran_ta_status_enum(val)
	if val == "APPROVED" || val == "REJECTED" {
		now := time.Now()
		pendaftaran.DecidedAt = now
	}

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []models.PendaftaranTA{pendaftaran}, nil
}

func (repo *PendaftaranRepo) SetTimeInterview(id string, val time.Time) (output []models.PendaftaranTA, err error) {
	var pendaftaran models.PendaftaranTA
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.InterviewAt = val
	pendaftaran.Status = models.Pendaftaran_ta_status_enum("INTERVIEW")

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []models.PendaftaranTA{pendaftaran}, nil
}

func (repo *PendaftaranRepo) GetPendaftaranByPeriode(idPeriode string) (output []models.PendaftaranTA, err error) {
	var periode []models.PendaftaranTA
	result := repo.DBRead.Where("id_periode = ?", idPeriode).Find(&periode)
	if result.Error != nil {
		return nil, err
	}
	return periode, nil
}
