package usecase

import (
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
)

type PendaftaranUsecase interface {
	AddPendaftaran(pendaftaran models.PendaftaranTA) (models.PendaftaranTA, error)
	GetPendaftaran() (output []models.PendaftaranTA, err error)
	DeletePendaftaran(id string) (output models.PendaftaranTA, err error)
	GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error)
	UpdateStatusPendaftaran(id string, val string) (output []models.PendaftaranTA, err error)
	UpdateWaktuInterview(id string, val time.Time) (output []models.PendaftaranTA, err error)
	GetPendaftaranByPeriode(id string) (output []models.PendaftaranTA, err error)
	GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error)
}

type PendaftaranUc struct {
	pendaftaranRepo PendaftaranRepository
}

func NewPendaftaranUseCase(pendaftaranRepo PendaftaranRepository) *PendaftaranUc {
	return &PendaftaranUc{
		pendaftaranRepo: pendaftaranRepo,
	}
}

func (uc *PendaftaranUc) AddPendaftaran(pendaftaran models.PendaftaranTA) (models.PendaftaranTA, error) {
	newPendaftaran, err := uc.pendaftaranRepo.NewPendaftaran(pendaftaran)

	if err != nil {
		return newPendaftaran, err
	}

	return newPendaftaran, nil

}

func (uc *PendaftaranUc) GetPendaftaran() (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaran()
}

func (uc *PendaftaranUc) DeletePendaftaran(id string) (output models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.DeletePendaftaran(id)
}

func (uc *PendaftaranUc) GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaranByIdDosen(id)
}

func (uc *PendaftaranUc) UpdateStatusPendaftaran(id string, val string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.SetStatus(id, val)
}

func (uc *PendaftaranUc) UpdateWaktuInterview(id string, val time.Time) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.SetTimeInterview(id, val)
}

func (uc *PendaftaranUc) GetPendaftaranByPeriode(id string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaranByPeriode(id)
}

func (uc *PendaftaranUc) GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error) {
	return uc.pendaftaranRepo.GetPendaftaranById(id)
}
