package entity

import "time"

type PendaftaranSidSem struct {
	ID              string    `gorm:"type:uuid;primaryKey;default:gen_random_uuid()" json:"id"`
	PendaftaranTAID string    `gorm:"type:uuid" json:"pendaftaranTAId"`
	Tipe            string    `json:"tipe"`
	Ditolak         bool      `json:"ditolak"`
	Lulus           bool      `json:"lulus"`
	JudulProposal   string    `json:"judul_proposal"`
	Deskripsi       string    `json:"deskripsi"`
	WaktuMulai      time.Time `gorm:"type:timestamptz" json:"waktuMulai"`
	WaktuSelesai    time.Time `gorm:"type:timestamptz" json:"waktuSelesai"`
	RuanganID       *string   `gorm:"type:uuid" json:"ruanganId"`
	IdPeriode       string    `json:"id_periode"`
	Link            string
}

func (PendaftaranSidSem) TableName() string {
	return "pendaftaran_sidsem"
}

type PendaftaranSidSemBerkas struct {
	ID              string         `gorm:"type:uuid;primaryKey;default:gen_random_uuid()" json:"id"`
	PendaftaranTAID string         `gorm:"type:uuid" json:"pendaftaranTAId"`
	Tipe            string         `json:"tipe"`
	Ditolak         bool           `json:"ditolak"`
	Lulus           bool           `json:"lulus"`
	JudulProposal   string         `json:"judul_proposal"`
	Deskripsi       string         `json:"deskripsi"`
	WaktuMulai      time.Time      `gorm:"type:timestamptz" json:"waktuMulai"`
	WaktuSelesai    time.Time      `gorm:"type:timestamptz" json:"waktuSelesai"`
	RuanganID       *string        `gorm:"type:uuid;null" json:"ruanganId"`
	IdPeriode       string         `json:"id_periode"`
	BerkasSidSem    []BerkasSidSem `json:"berkas"`
}

type PendaftaranSidSemBerkasReq struct {
	IDMahasiswa     string         `json:"id_mahasiswa"`
	PendaftaranTAID string         `gorm:"type:uuid" json:"pendaftaran_ta_id"`
	Tipe            string         `json:"tipe"`
	JudulProposal   string         `json:"judul_proposal"`
	Deskripsi       string         `json:"deskripsi"`
	BerkasSidSem    []BerkasSidSem `json:"berkas"`
}

type PendaftaranSidSemRequest struct {
	ID           string
	Nim          string
	Nama         string
	WaktuMulai   time.Time
	WaktuSelesai time.Time
	Tipe         string
	Link         string
}

type RekapPendaftaranRequest struct {
	ID          string `json:"id"`
	Nim         string `json:"nim"`
	IdMahasiswa string `json:"id_mahasiswa"`
	Nama        string `json:"nama_mahasiswa"`
	Dosbing     string `json:"nama_dosbing"`
	Tipe        string `json:"tipe"`
	Ditolak     bool   `json:"status"`
}
type SeminarPendaftaranSidSemRequest struct {
	ID              string
	Nim             string
	Nama            string
	Email           string
	Topik           string
	Deskripsi       string
	DosenPembimbing string
	DosenPenguji    string
	WaktuMulai      time.Time
	WaktuSelesai    time.Time
	Tipe            string
	Link            string
	Ruang           string
}

type SidangPendaftaranSidSemRequest struct {
	ID              string
	Nim             string
	Nama            string
	Email           string
	Topik           string
	Deskripsi       string
	DosenPembimbing string
	DosenPenguji    string
	DosenPenguji2   string
	WaktuMulai      time.Time
	WaktuSelesai    time.Time
	Tipe            string
	Link            string
	Ruang           string
}

type ParamValues struct {
	Limit  int
	Offset int
}

type EditRuanganRequest struct {
	Id        string `json:"id"`
	IdRuangan string `json:"id_ruangan"`
}

type EditWaktuRequest struct {
	Id           string    `json:"id"`
	WaktuMulai   time.Time `json:"waktuMulai"`
	WaktuSelesai time.Time `json:"waktuSelesai"`
}

type EditDosenPengujiSidangRequest struct {
	IdSidSem string `json:"id"`
	IdDosen1 string `json:"id_dosen1"`
	IdDosen2 string `json:"id_dosen2"`
}

type EditDosenPengujiSeminarRequest struct {
	IdSidSem string `json:"id"`
	IdDosen1 string `json:"id_dosen1"`
}

type AddPendaftaranSidSemRequest struct {
	IdPendaftaranTA string `json:"id_pendaftaran_ta"`
	Tipe            string `json:"tipe"`
	IdPeriode       string `json:"id_periode"`
}

type EditLulusRequest struct {
	Id    string
	Lulus bool
}

type EditDitolakRequest struct {
	Id      string `json:"id"`
	Ditolak bool   `json:"ditolak"`
}

type UpdateRuanganReq struct {
	NamaRuangan string `json:"nama_ruangan"`
	IdSidSem    string `json:"id_sidsem"`
}

type UpdateJadwalReq struct {
	WaktuMulai string `json:"waktu_mulai"`
	IdSidSem   string `json:"id_sidsem"`
}

type UpdateDosenReq struct {
	Dosujis  []Dosuji `json:"dosen_uji"`
	IdSidSem string   `json:"id_sidsem"`
}

type Placeholders struct {
	Nama         string `json:"nama"`
	Nim          string `json:"nim"`
	JalurPilihan string `json:"jalur_pilihan"`
	Judul        string `json:"judul"`
	Dosbing      string `json:"dosbing"`
}

type NameTopik struct {
	Nama         string
	Nim          string
	JalurPilihan string
	Judul        string
}

type DosbingName struct {
	Nama string
}

type InfoTopikMahasiswa struct {
	IDMahasiswa  string `json:"id_mahasiswa"`
	Nama         string `json:"nama"`
	Email        string `json:"email"`
	JalurPilihan string `json:"jalur_pilihan"`
	Judul        string `json:"judul"`
	Deskripsi    string `json:"deskripsi"`
}

type RemainingDetailSidSem struct {
	Tipe       string    `json:"tipe"`
	WaktuMulai time.Time `json:"waktu_mulai"`
	RuanganID  string    `json:"ruangan_id"`
	Ditolak    bool      `json:"ditolak"`
}

type DetailSidSemByMahasiswa struct {
	IDMahasiswa  string    `json:"id_mahasiswa"`
	Nama         string    `json:"nama"`
	Email        string    `json:"email"`
	JalurPilihan string    `json:"jalur_pilihan"`
	Judul        string    `json:"judul"`
	Deskripsi    string    `json:"deskripsi"`
	DosbingName  string    `json:"dosbing_name"`
	Tipe         string    `json:"tipe"`
	WaktuMulai   time.Time `json:"waktu_mulai"`
	NamaRuangan  string    `json:"nama_ruangan"`
	Ditolak      bool      `json:"ditolak"`
}

type DetailSidSemTA struct {
	IDMahasiswa   string         `json:"id_mahasiswa"`
	Nama          string         `json:"nama"`
	Email         string         `json:"email"`
	JalurPilihan  string         `json:"jalur_pilihan"`
	DosbingName   string         `json:"dosbing_name"`
	DosujiName    []Dosuji       `json:"dosuji_name"`
	Tipe          string         `json:"tipe"`
	JudulProposal string         `json:"judul_proposal"`
	Deskripsi     string         `json:"deskripsi"`
	Berkas        []BerkasSidSem `json:"berkas_sidsem"`
	JadwalSidang  time.Time      `json:"jadwal_sidang"`
	Tempat        string         `json:"tempat"`
	Status        bool           `json:"status"`
}

type TipeJudulDesc struct {
	JudulProposal string `json:"judul_proposal"`
	Deskripsi     string `json:"deskripsi"`
}

type JadwalStatus struct {
	WaktuMulai string `json:"waktu_mulai"`
	Ditolak    string `json:"ditolak"`
}

type Ruangan struct {
	ID   string `json:"id"`
	Nama string `json:"nama"`
}

type Dosuji struct {
	ID   string `json:"id"`
	Nama string `json:"nama"`
}

type PengujiSidsem struct {
	ID       string `json:"id"`
	SidsemID string `json:"sidsem_id"`
	DosenID  string `json:"dosen_id"`
}
