package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type MahasiswaPendaftaranSidSemHandler struct {
	pendaftaranSidSemUsecase usecase.PendaftaranSidSemUsecase
}

func NewMahasiswaPendaftaranSidSemHandler(cfg config.PendaftaranSidSemTransportConfig) *MahasiswaPendaftaranSidSemHandler {
	pendaftaranSidSemRepository := repository.NewPendaftaranSidSemRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaranSidSemUsecase := usecase.NewPendaftaranSidSemUseCase(pendaftaranSidSemRepository)

	return &MahasiswaPendaftaranSidSemHandler{
		pendaftaranSidSemUsecase: pendaftaranSidSemUsecase,
	}

}

func (t *MahasiswaPendaftaranSidSemHandler) MountMahasiswa(group *echo.Group) {
	group.GET("/get-placeholders", t.GetPlaceholders)
	group.POST("/pendaftaran-sidsem", t.AddPendaftaran)
	group.GET("/is-registered-sidsem", t.IsRegisteredSidsem)
	group.GET("/get-sidsem-detail", t.GetSidSemDetailByMahasiswaId)
}

func (t *MahasiswaPendaftaranSidSemHandler) AddPendaftaran(c echo.Context) error {
	pendaftaranSidSemBerkas := new(entity.PendaftaranSidSemBerkasReq)
	if err := c.Bind(&pendaftaranSidSemBerkas); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	newPendaftaran, err := t.pendaftaranSidSemUsecase.AddPendaftaran(*pendaftaranSidSemBerkas)

	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran added successfully", newPendaftaran))
}

func (t *MahasiswaPendaftaranSidSemHandler) GetPlaceholders(c echo.Context) error {
	id_mahasiswa := c.QueryParam("id_mahasiswa")

	placeholders, err := t.pendaftaranSidSemUsecase.GetPlaceholders(id_mahasiswa)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Placeholders retrieved successfully", placeholders))
}

func (t *MahasiswaPendaftaranSidSemHandler) IsRegisteredSidsem(c echo.Context) error {
	tipe := c.QueryParam("tipe")
	payload, ok := c.Get("authPayload").(utils.Payload)
	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	result, err := t.pendaftaranSidSemUsecase.IsRegisteredSidsem(payload.ID, tipe)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Is Registered retrieved successfully", result))
}

func (t *MahasiswaPendaftaranSidSemHandler) GetSidSemDetailByMahasiswaId(c echo.Context) error {
	tipe := c.QueryParam("tipe")
	payload, ok := c.Get("authPayload").(utils.Payload)
	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	result, err := t.pendaftaranSidSemUsecase.GetSidSemDetailByMahasiswaId(payload.ID, tipe)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Sidsem Detail retrieved successfully", result))
}
