package repository

import (
	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gorm.io/gorm"
)

type TopicAllocationRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewTopicAllocationRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *TopicAllocationRepo {
	return &TopicAllocationRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *TopicAllocationRepo) NewTopic(reqData entity.TopikPostPutReqData) (output entity.TopikPostPutReqData, err error) {
	topic := entity.Topik{
		ID:        	uuid.New().String(),
		Judul:     	reqData.Judul,
		Deskripsi: 	reqData.Deskripsi,
		PengajuID: 	reqData.PengajuID,
	}

	result := repo.DBWrite.Create(&topic)
	if result.Error != nil {
		return entity.TopikPostPutReqData{}, result.Error
	}

	return reqData, nil
}


func (repo *TopicAllocationRepo) GetTopic(page int, limit int, search string, id string) (output entity.GetAllReturnType, err error) {
	var topics []entity.Topik

	result := repo.DBRead.Preload("Pengguna")

	if search != "" {
		result = result.Where("judul LIKE ?", "%"+search+"%")
	}

	if id != "" {
		result = result.Where("pengaju_id = ?", id)
	}

	if page != 0 {
		result = result.Offset((page - 1) * limit)
	}

	if limit != 0 {
		result = result.Limit(limit)
	}

	result = result.Find(&topics)
	if result.Error != nil {
		return entity.GetAllReturnType{}, result.Error
	}

	var count int64
	res := result.Count(&count)
	if res.Error != nil {
		return entity.GetAllReturnType{}, res.Error
	}

	var maxPage int
	
	if limit == 0 || count == 0 {
		maxPage = 1
	} else {
		maxPage = int(count) / limit
	}

	data := entity.GetAllReturnType {
		DaftarTopikData: topics,
		MaxPage: maxPage,
	}

	return data, nil
}

func (repo *TopicAllocationRepo) GetTopicByID(ID string) (output entity.Topik, err error) {
	var topic entity.Topik

	result := repo.DBRead.Preload("Pengguna").Where("id = ?", ID).First(&topic)
	if result.Error != nil {
		return entity.Topik{}, result.Error
	}

	return topic, nil
}

func (repo *TopicAllocationRepo) UpdateTopic(reqData entity.TopikPostPutReqData, id string) (output entity.TopikPostPutReqData, err error) {
	var topic entity.Topik
	topicResult := repo.DBWrite.First(&topic, "id = ?", id)
	if topicResult.Error != nil {
		return entity.TopikPostPutReqData{}, topicResult.Error
	}

	topic.Judul = reqData.Judul
	topic.Deskripsi = reqData.Deskripsi

	result := repo.DBWrite.Save(&topic)
	if result.Error != nil {
		return entity.TopikPostPutReqData{}, result.Error
	}

	return reqData, nil
}


func (repo *TopicAllocationRepo) DeleteTopic(ID string) (err error) {
	var topic entity.Topik

	result := repo.DBWrite.Where("id = ?", ID).Delete(&topic)

	if result.Error != nil {
		return result.Error
	}

	return nil
}
