package usecase

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TopicUsecase interface {
	AddTopic(topic []entity.TopikPostPutReqData) ([]entity.TopikPostPutReqData, error)
	GetTopic(param utils.PageLimitSearchID) (entity.GetAllReturnType, error)
	GetTopicByID(param string) (entity.Topik, error)
	UpdateTopic(topic entity.TopikPostPutReqData, id string) (entity.TopikPostPutReqData, error)
	DeleteTopic(param string) (error)
}

type TopicUc struct {
	topicAllocationRepo TopicAllocationRepo
}

func NewTopicUc(topicAllocationRepo TopicAllocationRepo) *TopicUc {
	return &TopicUc{
		topicAllocationRepo: topicAllocationRepo,
	}
}

func (uc *TopicUc) AddTopic(topics []entity.TopikPostPutReqData) ([]entity.TopikPostPutReqData, error) {
	for _, topic := range topics {
		_, err := uc.topicAllocationRepo.NewTopic(topic)
		if err != nil {
			return nil, err
		}
	}

	return topics, nil
}

func (uc *TopicUc) GetTopic(param utils.PageLimitSearchID) (entity.GetAllReturnType, error) {
	return uc.topicAllocationRepo.GetTopic(param.Page, param.Limit, param.Search, param.ID)
}

func (uc *TopicUc) GetTopicByID(param string) (entity.Topik, error) {
	return uc.topicAllocationRepo.GetTopicByID(param)
}

func (uc *TopicUc) UpdateTopic(topic entity.TopikPostPutReqData, id string) (entity.TopikPostPutReqData, error) {
	updatedTopic, err := uc.topicAllocationRepo.UpdateTopic(topic, id)

	if err != nil {
		return entity.TopikPostPutReqData{}, err
	}

	return updatedTopic, nil
}

func (uc *TopicUc) DeleteTopic(param string) (error) {
	err := uc.topicAllocationRepo.DeleteTopic(param)

	if err != nil {
		return err
	}

	return nil
}